/* ============================================================
 * Date  : 2008-1-20
 * Copyright 2008 Palm, Inc. All rights reserved.
 * ============================================================ */

#ifndef DOCCOMMANDER_H
#define DOCCOMMANDER_H

#include "PDFRenderer.h"
#define MAX_SHARED_BUF 11

struct SharedBuffer
{
    int shmid;
    size_t size;
    void *buffer;
};

class DocCommander
{
public:
    DocCommander(char *filename, int id);
    ~DocCommander();
  
    //Returns wether initialization had an error
    bool HasInitError() {return mInitError;}

    int   Initialize();    
    void  SetPassword(char *password);
    void  RegisterBuffer(int shmid, size_t size);
    int   NextPage();
    bool  NeedsPassword();
    int   PreviousPage();
    int   JumpToPage(int page);
    void  SetZoom(double inZoom);
    double GetZoom();
    void  SetRotation(int inRotation);
    int   GetRotation();
    int   GetCurrentPageNumber();
    int   GetPageCount();
    void  GetPageDimensions(int &width, int &height);
    int   RenderToBuffer(int x, int y, int width, int height, int shmid);
    int   HasLinkAtLocation(int x, int y, char *, bool);
    void OutlineRoot();
    char *GetOutlineNodeText();
    void OutlineGotoNextSibling();
    void OutlineGotoChild();
    bool OutlineHasNextSibling();
    bool OutlineHasChild();
    int GetOutlineLinkPageNumber();
    void CaseInsensitiveTextSearch(char *needle,int startpage, int endpage,
				   void (*RecordResult)(int page, int x0, int y0, int x1, int y1, int contextlen, char *context, void *), 
				   void *userdata);


    //for testing?
    void DumpCurrentPageText();
    void TranslateScreenPoint(int &x, int &y);
    void TranslateScreenDelta(int &x, int &y);
private:
    
    PDFRenderer  mRenderer;
    bool mNeedsPassword;
    int mID;
    PDFOutline *mOutline;
    SharedBuffer mRegisteredBuffers[MAX_SHARED_BUF];
    bool mInitError;
};

#endif
