#include <stdio.h>
#include "PDFPage.h"
#include "PDF.h"
using namespace std;


PDFPage::PDFPage(PDF *parent, int pagenumber):
    mPage(NULL),
    mParent(parent),
    mPageNumber(pagenumber)
{
    fz_error *error;
    fz_obj *pageobj;
    
    pageobj = pdf_getpageobject(mParent->mSrcPages, mPageNumber - 1);
    error = pdf_loadpage(&mPage, mParent->mSrc, pageobj);
    if (error)
    {
        if(mPage != NULL)
            pdf_droppage(mPage);
        mPage = NULL;
    }
}

PDFPage::~PDFPage()
{
    pdf_droppage(mPage);
    mPage = NULL;
}

bool PDFPage::Loaded()
{
    return mPage != NULL;
}

int PDFPage::GetPageNumber()
{
    return mPageNumber;
}

float PDFPage::GetWidth()
{
    if(mPage)
        return mPage->mediabox.x1 - mPage->mediabox.x0;
    return 0;
}

float PDFPage::GetHeight()
{
    if(mPage)
        return mPage->mediabox.y1 - mPage->mediabox.y0;
    return 0;
}

int PDFPage::GetRotation()
{
    if(mPage)
        return mPage->rotate;
    return 0;
}

void PDFPage::CaseInsensitiveSearchPageText(char *needle, int needlelength, int *badchar, int* goodsuff,
					    void (*RecordResult)(int page, int x0, int y0, int x1, int y1, int contextlen, char *context, void *), void *userdata)
{
    pdf_textline *thisline=NULL;
    pdf_textline *lastline=NULL;
    fz_matrix ctm;
    fz_error *error;
    int i;
    ctm = fz_identity();

    error = pdf_loadtextfromtree(&thisline, mPage->tree, ctm);
    if(error)
      printf("There was an error getting text from tree!\n");

    int curoffset=0;
    int turbou=0;
    int turbov =0;
    int shift = needlelength;
    pdf_textline *compareline;
    //i=needlelength-1;
    int j;
    int c;
    char comp;
    int badcharshift, turboshift;
    char tmp[1024];
    int index, resultx0, resultx1, resulty0, resulty1 = 0;
    while(thisline != NULL)
    {
        bool straddle = false;
        if(thisline->len == 0)
        {
            thisline = thisline->next;	
            if(thisline == NULL)
                continue;
        }
        if(curoffset+needlelength > thisline->len)//the search straddles the line
        {
            lastline = thisline;
            thisline = thisline->next;
            straddle = true;
            if(thisline == NULL) //we reached the end of page and thus there can be no match
                continue;
        }
        i = needlelength-1;
        while(i>=0)
        {
            if(straddle && curoffset+i >= lastline->len)//we are on farther line
            {
                j = (curoffset+i)%(lastline->len);
                compareline = thisline;
            }
            else 
            {
                j = curoffset+i;
                if(straddle)
                {
                    compareline=lastline;
                }
                else
                {
                    compareline=thisline;
                }
            }
            c = compareline->text[j].c;
            if(c>=0 && c<128)
            {
                comp = mytolower((char)c);
		//if we are at the end of the line and we are comparing space... skip it
		if(straddle && j == lastline->len && needle[i] == ' ')
		  i--;
                if(comp != needle[i])
                    break;
                i--;
                if(turbou!=0 && i == needlelength -1 -shift)
                    i-=turbou;
            }
            else //not ascii character, not handled
            {
                c=128;
                break;
            }
        }
        if(i<0)
        {//We have matched!! What now?
	    index=0;
            if(straddle)
	    {
	      resultx0 = lastline->text[curoffset].bbox.x0;
	      resulty0 = lastline->text[curoffset].bbox.y0;
	      j = (curoffset +needlelength)%lastline->len;
	      for(index=0;index<lastline->len && index<1024;index++)
	      {
		if(lastline->text[index].c <256)
		{
		  tmp[index] = (char)lastline->text[index].c;
		}
		else
		{
		    tmp[index] = ' ';
		}
	      }
	      tmp[index] = '\n';
	    }
	    else
	    {
	      resultx0 = thisline->text[curoffset].bbox.x0;
	      resulty0 = thisline->text[curoffset].bbox.y0;
	      j = curoffset+needlelength;
	    }
	    resultx1 = thisline->text[j].bbox.x1;
	    resulty1 = thisline->text[j].bbox.y1;
	    int stat = index ? index+1:0;
	    for(index=0;index<thisline->len && index<1024;index++)
	    {
	      if(thisline->text[index].c <256)
	      {
		tmp[stat+index] = (char)thisline->text[index].c;
	      }
	      else
	      {
		tmp[stat+index] = ' ';
	      }
	    }
	    tmp[stat+index] = '\0';
	    RecordResult(GetPageNumber(),resultx0,resulty0,resultx1,resulty1,strlen(tmp),tmp, userdata);
            shift = goodsuff[0];
            turbou = needlelength - shift;
            curoffset += shift;

        }
        else
        {
            turbov = needlelength -1 -i;
            turboshift = turbou-turbov;
            badcharshift = badchar[c]-needlelength+1+i;
            shift = MAX(turboshift, badcharshift);
            shift = MAX(shift, goodsuff[i]);

            if(shift == goodsuff[i])
                turbou = MIN(needlelength-shift, turbov);
            else
            {
                if(turboshift < badcharshift)
                    shift = MAX(shift, turbou+1);
                turbou=0;
            }
            curoffset += shift;
        }
        if((!straddle) && curoffset >= thisline->len) // if we were not straddled and curoffset jump line update
        {
            lastline = thisline;
            thisline = thisline->next;
            if(thisline == NULL)
                continue;
            curoffset = curoffset%(thisline->len);
        }
        else if(straddle)//if we are straddled we got a lot of handling
        {
            if(curoffset < lastline->len)//if we didn't shift to the next line lets undo something so straddle case works on next iteration
            {
                thisline=lastline;
            }
            else//otherwise this line and lastline are correct just offset needs modding
            {
                curoffset = curoffset%lastline->len;
            }
        }
    }
}


void PDFPage::DebugPageText()
{
  pdf_textline *line;
  fz_matrix ctm;
  fz_error *error;
  ctm = fz_identity();

  error = pdf_loadtextfromtree(&line, mPage->tree, ctm);
  if(error)
    printf("There was an error getting text from tree!\n");

  DebugTextLine(line);
}

void PDFPage::DebugTextLine(pdf_textline *line)
{
  char buf[10];
  int c, n, k, i;
  
  for (i = 0; i < line->len; i++)
    {
      c = line->text[i].c;
      if (c < 128)
	printf("%c",((char)c));
      else
	{
	  
	  n = runetochar(buf, &c);
	  //cout << "ELSE!!!! has "<< n << " rune chars?"  <<endl;
	  for (k = 0; k < n; k++)
	    printf("%c",((char)buf[k]));
	    
	}
    }
  printf("\n");
  if(line->next != NULL)
    {
      DebugTextLine(line->next);
    }
}
