
#ifndef PDFRENDER_H
#define PDFRENDER_H

#include "PDF.h"
#include "mustd.h"

class PDF;

///PDFRenderer class
///public PDF
/// The PDFRenderer class inherits PDF class with the intent of providing functionality to render the underlying
/// pdf file.  For the class to do anything, you must call SetRenderingMechanism and pass in
/// the appropriate callback function.  If the callback function is null, we go through the motions of rendering
/// but do not callback for each pixel.

class PDFRenderer: public PDF
{
public:
    ///PDFRenderer(char *filename) -- constructor
    /// filename is the ascii path of the file you wish to open.
    PDFRenderer(char *filename);
    ~PDFRenderer();

    /// SetRenderingMechanism takes the function that you intend to use for rendering
    /// inFunc(int x, int y, unsigned char red, unsigned char green, unsigned char blue)
      void SetRenderingMechanism(void (*inFunc)(int, int, int, unsigned char, unsigned char, unsigned char,void *));

    /// Zoom interface.  Takes a floating point argument corresponding to the zoom -- for example 1= no zoom.  .5 = 50%, 2 = 200%
    void SetZoom(double inZoom);
    double GetZoom();

    /// Rotation interface: Takes degree argument corresponding to rotation -- 0,90,180... etc..
    void SetRotation(int inRotation);
    int GetRotation();

    /// RenderCurrentPage renders the page via the call back function set in SetRenderingMechanism
    /// (x,y) is the location on page to place "center" of window.  (width,height) are the dimension of the window
    /// mBuf is for user convenience.  We can pass a pointer to memory buffer on to your rendering func
    /// So if your buffer is from a class var from a non global context you can pass to your global
    /// function
    int RenderCurrentPage(int x, int y, int width, int height, void *mBuf);

    /// Because of zoom and rotation matrices being applied to the image prior to use input,
    /// these two functions are intended to take screen coordinates and "unapply" the matrices.
    /// For example, user clicks at 10,10 -- TranslateScreenPoint figures out what 10,10 corresponds to
    /// in page coordinates taking into account zoom and rotation, but not window center.
    fz_point TranslateScreenPoint(int x,int y);
    fz_point TranslateScreenDelta(int x,int y);



private:
    fz_renderer *mGC;
    double mDrawZoom;
    int mDrawRotation;
    //keeping these around since if user needs to ask us
    // a question about where the mouse is or translate relative to absolute or back we need these saved
    // from last transition
    fz_matrix mCtm;
    fz_point mCurrentTopLeft;
    fz_irect mBbox;

    void (* DrawPixel)(int , int , int, unsigned char , unsigned char , unsigned char, void * );
};

#endif
